@@ -1,4 +1,4 @@
-# Copyright (C) 2003,2006,2007 Slaven Rezic. All rights reserved.
+# Copyright (C) 2003,2006,2007,2010,2013 Slaven Rezic. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
@@ -6,18 +6,21 @@ package TkTest;
 
 use strict;
 use vars qw(@EXPORT @EXPORT_OK $eps $VERSION);
-$VERSION = '4.008'; # was: sprintf '4.%03d', q$Revision: #3 $ =~ /\D(\d+)\s*$/;
+$VERSION = '4.010';
 
 use base qw(Exporter);
-@EXPORT    = qw(is_float is_float_pair check_display_harness);
-@EXPORT_OK = qw(catch_grabs wm_info);
+@EXPORT    = qw(is_float is_float_pair checked_test_harness);
+@EXPORT_OK = qw(catch_grabs wm_info set_have_fixed_font with_fixed_font);
 
 sub _is_in_path ($);
 
 use POSIX qw(DBL_EPSILON);
 $eps = DBL_EPSILON;
 
-sub check_display_harness () {
+sub checked_test_harness ($@) {
+    my($skip_test_dir, @test_harness_args) = @_;
+
+    require ExtUtils::Command::MM;
     # In case of cygwin, use'ing Tk before forking (which is done by
     # Test::Harness) may lead to "remap" errors, which are normally
     # solved by the rebase or rebaseall utilities.
@@ -25,22 +28,33 @@ sub check_display_harness () {
     # Here, I just skip the DISPLAY check on cygwin to not force users
     # to run rebase.
     #
-    return if $^O eq 'cygwin' || $^O eq 'MSWin32';
-
-    eval q{
+    if (!($^O eq 'cygwin' || $^O eq 'MSWin32')) {
+	eval q{
            use blib;
            use Tk;
         };
-    die "Strange: could not load Tk library: $@" if $@;
+	die "Strange: could not load Tk library: $@" if $@;
+    }
 
-    if (defined $Tk::platform && $Tk::platform eq 'unix') {
+    if (defined $Tk::platform && $Tk::platform eq 'unix') { # undef for cygwin+MSWin32, because Tk not yet loaded
+	my $skip_all_test;
 	my $mw = eval { MainWindow->new() };
 	if (!Tk::Exists($mw)) {
-	    warn "Cannot create MainWindow (maybe no X11 server is running or DISPLAY is not set?)\n$@\n";
-	    exit 0;
+	    $skip_all_test = "skip_all_mw.t";
+	} else {
+	    my $l = eval { $mw->Label(-text => "test") }; # will allocate a font, which may fail
+	    if (!Tk::Exists($l)) {
+		$skip_all_test = "skip_all_font.t";
+	    }
+	}
+	if ($skip_all_test) {
+	    local @ARGV = "$skip_test_dir/$skip_all_test";
+	    return ExtUtils::Command::MM::test_harness(@test_harness_args);
 	}
 	$mw->destroy;
     }
+
+    return ExtUtils::Command::MM::test_harness(@test_harness_args);
 }
 
 sub is_float ($$;$) {
@@ -192,6 +206,24 @@ sub wm_info ($) {
     );
 }
 
+{
+    my $have_fixed_font;
+
+    sub set_have_fixed_font ($) {
+	$have_fixed_font = shift;
+    }
+
+    sub with_fixed_font (&) {
+	my $testcode = shift;
+	local $Test::Builder::Level = $Test::Builder::Level + 1;
+    SKIP:
+	{
+	    Test::More::skip("fixed courier font not available", 1) if !$have_fixed_font;
+	    $testcode->();
+	}
+    }
+}
+
 # REPO BEGIN
 # REPO NAME is_in_path /home/e/eserte/work/srezic-repository 
 # REPO MD5 e18e6687a056e4a3cbcea4496aaaa1db
